/*
 * Copyright (c) 2018-2020. Developed by Hedgecode.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.hedgecode.chess.pgn.token;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.hedgecode.chess.pgn.entity.DetailMove;
import org.hedgecode.chess.pgn.entity.Moves;
import org.hedgecode.chess.position.ParseException;

/**
 * MoveToken
 *
 * @author Dmitry Samoshin aka gotty
 */
public class MoveToken implements Token<Moves> {

    private static final String MOVE_REGEX = "^\\s*(([0-9]+)\\s*(\\.|\\.{3})\\s*)*([PNBRQK]?[a-hx1-8\\-O]+(=[NBRQ])?([+#])?)";
    private static final Pattern MOVE_PATTERN = Pattern.compile(MOVE_REGEX);
    private static final int PLY_GROUP = 2, DOT_GROUP = 3, MOVE_GROUP = 4;
    private static final String DOT_BLACK = "...";

    @Override
    public int token(Moves moves, String pgn) throws ParseException {
        Matcher matcher = MOVE_PATTERN.matcher(pgn);
        if (!matcher.find()) {
            throw new ParseException("parse.pgn.incorrect.move", pgn);
        } else {
            boolean isBlackMove = isBlackMove(matcher.group(DOT_GROUP));
            String plyStr = matcher.group(PLY_GROUP);
            int ply = plyStr != null
                    ? Integer.parseInt(plyStr) * 2 - (isBlackMove ? 0 : 1)
                    : moves.currentMove().ply() + 1;
            String move = matcher.group(MOVE_GROUP);
            moves.addMove(
                    new DetailMove(ply, move)
            );
        }
        return matcher.group().length();
    }

    private boolean isBlackMove(String dotGroup) {
        return dotGroup == null || dotGroup.equals(DOT_BLACK);
    }

}
