/*
 * Copyright (c) 2018-2020. Developed by Hedgecode.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.hedgecode.chess.pgn.token;

import java.util.regex.Pattern;

import org.hedgecode.chess.pgn.entity.Game;
import org.hedgecode.chess.position.ParseException;

/**
 * Tokenizer
 *
 * @author Dmitry Samoshin aka gotty
 */
public class Tokenizer {

    enum Type {

        TAG   ( new TagToken()   ),
        EMPTY ( new EmptyToken() ),
        MOVES ( new MovesToken() );

        private Token<Game> token;

        public Token<Game> token() {
            return token;
        }

        Type(Token<Game> token) {
            this.token = token;
        }
    }

    private static final Pattern TAG_PATTERN = Pattern.compile("^\\[[^]]+\\]$", Pattern.MULTILINE);
    private static final Pattern EMPTY_PATTERN = Pattern.compile("^\\s*$", Pattern.MULTILINE);
    private static final Pattern MOVES_PATTERN = Pattern.compile("^\\s*([^\\s])", Pattern.MULTILINE);

    private final String pgn;

    private Token<Game> token;
    private String tokenPgn;

    public Tokenizer(String pgn) {
        this.pgn = pgn;
        this.tokenPgn = pgn;
    }

    public boolean hasToken() {
        assignToken();
        return token != null;
    }

    public void token(Game game) throws ParseException {
        if (token == null) {
            throw new ParseException("parse.pgn.null.token");
        }
        int index = token.token(game, tokenPgn);
        tokenPgn = tokenPgn.substring(index);
        token = null;
    }

    public String sourcePgn() {
        return pgn;
    }

    private void assignToken() {
        token = null;
        if (!tokenPgn.isEmpty()) {
            if (TAG_PATTERN.matcher(tokenPgn).find()) {
                token = Type.TAG.token();
            } else if (EMPTY_PATTERN.matcher(tokenPgn).find()) {
                token = Type.EMPTY.token();
            } else if (MOVES_PATTERN.matcher(tokenPgn).find()) {
                token = Type.MOVES.token();
            }
        }
    }

}
