/*
 * Copyright (c) 2018-2019. Developed by Hedgecode.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.hedgecode.chess.img.board;

import java.awt.image.BufferedImage;
import java.util.HashMap;
import java.util.Map;

import org.hedgecode.chess.img.AbstractImageLoader;
import org.hedgecode.chess.img.FilenameUtils;
import org.hedgecode.chess.img.ImageConstants;
import org.hedgecode.chess.img.ImageException;
import org.hedgecode.chess.img.ImageFilter;

/**
 * Loader for chess board's set of images.
 *
 * @author Dmitry Samoshin aka gotty
 */
public class BoardLoader extends AbstractImageLoader {

    private static final String SQUARES_DIR = ImageConstants.RESOURCE_SQUARES_DIR;

    private static final String SQUARE_DARK = ImageConstants.DARK_SQUARE_FILENAME;
    private static final String SQUARE_LIGHT = ImageConstants.LIGHT_SQUARE_FILENAME;

    private static final String BOARD = ImageConstants.BOARD_FILENAME;

    private static final String[] BOARD_SQUARE_NAMES = { BOARD, SQUARE_DARK, SQUARE_LIGHT };

    private Map<String, Board> squareSetMap = new HashMap<>();

    @Override
    public Board load(String name) throws ImageException {
        Board board;
        if (Type.STATELESS.equals(loadType()) || !squareSetMap.containsKey(name)) {
            String boardPath = FilenameUtils.getFullPath(SQUARES_DIR, name);
            board = loadBoard(boardPath);
            if (Type.STATEFUL.equals(loadType())) {
                squareSetMap.put(name, board);
            }
        } else {
            board = squareSetMap.get(name);
        }
        return board;
    }

    @Override
    public void unload(String name) {
        if (Type.STATEFUL.equals(loadType()) && squareSetMap.containsKey(name)) {
            squareSetMap.remove(name);
        }
    }

    @Override
    protected void clear() {
        squareSetMap.clear();
    }

    private Board loadBoard(String boardPath) throws ImageException {
        Map<String, BufferedImage> images =
                loadImages(
                        boardPath,
                        new ImageFilter(BOARD_SQUARE_NAMES)
                );
        BufferedImage board = images.get(BOARD);
        SquarePair<BufferedImage> squarePair = SquarePair.create(
                images.get(SQUARE_DARK), images.get(SQUARE_LIGHT)
        );
        if (board == null && !squarePair.isFilled()) {
            throw new ImageException("image.incomplete.board.set", boardPath);
        }
        return board != null
                ? Board.create(board)
                : Board.create(squarePair);
    }

}
